@@ -1,5 +1,5 @@
 // ResourceTreeBrowser.java
-// $Id: ResourceTreeBrowser.java,v 1.30 1999/04/16 14:13:23 bmahe Exp $
+// $Id: ResourceTreeBrowser.java,v 1.30.4.1 1999/11/24 16:46:53 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1998.
 // Please first read the full copyright statement in file COPYRIGHT.html
  
@@ -59,7 +59,7 @@ import org.w3c.jigsaw.admin.RemoteResource;
 
 /**
  * A JTree used to manage RemoteResource.
- * @version $Revision: 1.30 $
+ * @version $Revision: 1.30.4.1 $
  * @author  Benot Mah (bmahe@w3.org)
  */
 public class ResourceTreeBrowser extends JTree 
@@ -110,7 +110,6 @@ public class ResourceTreeBrowser extends JTree
 	    }
 	}
     };
-
     /**
      * Our MouseListener
      */
@@ -198,8 +197,9 @@ public class ResourceTreeBrowser extends JTree
 	DataFlavor required = TransferableResourceCell.RESOURCE_CELL_FLAVOR;
 	try {
 	    if (tr.isDataFlavorSupported(required)) {
+		dropTargetDropEvent.acceptDrop(DnDConstants.ACTION_COPY);
 		ResourceCell cell = (ResourceCell)tr.getTransferData(required);
-		dropResource(dropTargetDropEvent, cell);
+		dropTargetDropEvent.dropComplete(dropResource(cell));
 	    } else {
 		dropTargetDropEvent.rejectDrop();
 	    }
@@ -214,30 +214,20 @@ public class ResourceTreeBrowser extends JTree
 
     /**
      * Drop a resource.
-     * @param dropTargetDropEvent The DropTargetDropEvent
      * @param cell The resource cell
      * @see org.w3c.jigadmin.editors.ResourceCell
      */
-    protected void dropResource(DropTargetDropEvent dropTargetDropEvent,
-				ResourceCell cell)
-    {
+    protected boolean dropResource(ResourceCell cell) {
 	RemoteResourceWrapper rrw = getSelectedResourceWrapper();
 	PropertyManager pm = PropertyManager.getPropertyManager();
 	try {
 	    if (! pm.isExtensible(rrw)) {
-		dropTargetDropEvent.rejectDrop();
-		return;
-	    } else if (cell.isFrame() || cell.isFilter()) {
-		if (pm.isEditable(rrw)) {
-		    dropTargetDropEvent.
-			acceptDrop(DnDConstants.ACTION_COPY);
-		    //popupresource
-		    popupResource(rrw);
-		    dropTargetDropEvent.
-			getDropTargetContext().dropComplete(true);
-		} else {
-		    dropTargetDropEvent.rejectDrop();
-		}
+		return false;
+	    } else if ((cell.isFrame() || cell.isFilter()) && 
+		       (pm.isEditable(rrw))) {
+		//popupresource
+		popupResource(rrw);
+		return true;
 	    } else if (((cell.isContainer() || cell.isResource()) && 
 			(! rrw.getResource().isIndexersCatalog())) 
 		       ||
@@ -245,30 +235,20 @@ public class ResourceTreeBrowser extends JTree
 			cell.isIndexer())) {
 		//must be droppped on a Container
 		if (rrw.getResource().isContainer()) {
-		    dropTargetDropEvent.
-			acceptDrop(DnDConstants.ACTION_COPY);
 		    //add the resource
 		    String identifier = getIdentifier(cell, rrw);
 		    if (identifier != null) {
 			TreePath path = getSelectionPath();
 			addResource(identifier, cell.toString(), 
 				    rrw, path);
-			dropTargetDropEvent.
-			    getDropTargetContext().dropComplete(true);
-			//expand the node if necessary
-		    } else {
-			dropTargetDropEvent.
-			    getDropTargetContext().dropComplete(true);
-		    }
-		} else {
-		    dropTargetDropEvent.rejectDrop();
+			return true;
+		    } 
 		}
-	    } else {
-		dropTargetDropEvent.rejectDrop();
 	    }
+	    return false;
 	} catch (RemoteAccessException ex) {
-	    dropTargetDropEvent.rejectDrop();
 	    Message.showErrorMessage(this, ex);
+	    return false;
 	}
     }
 
