@@ -1,5 +1,5 @@
 // HTTPFrame.java
-// $Id: HTTPFrame.java,v 1.51 1998/12/02 16:23:00 bmahe Exp $
+// $Id: HTTPFrame.java,v 1.63.4.6 2000/01/20 17:46:51 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1997.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -28,6 +28,9 @@ import org.w3c.tools.resources.ResourceException;
  */
 public class HTTPFrame extends ProtocolFrame {
 
+    public static final 
+    String STATE_CONTENT_LOCATION = "org.w3c.jigsaw.frames.HTTPFrame.cl";
+    
     private static final boolean debug = true;
 
     /**
@@ -85,6 +88,10 @@ public class HTTPFrame extends ProtocolFrame {
      * Attribute index - The index for the content type attribute.
      */
     protected static int ATTR_CONTENT_TYPE = -1 ;
+    /**
+     * Attribute index - The index for the charset attribute.
+     */
+    protected static int ATTR_CHARSET = -1 ;
     /**
      * Attribute index - The index for the content length attribute.
      */
@@ -173,10 +180,15 @@ public class HTTPFrame extends ProtocolFrame {
 	ATTR_CONTENT_ENCODING = AttributeRegistry.registerAttribute(cls,a);
 	// The content type attribute:
 	a = new MimeTypeAttribute("content-type"
-				  , MimeType.TEXT_PLAIN
+				  , null
 				  , Attribute.EDITABLE) ;
 	ATTR_CONTENT_TYPE = AttributeRegistry.registerAttribute(cls,a);
-	// The content length attribute:
+	// The Charset attribute:
+	a = new StringAttribute("charset"
+				, null
+				, Attribute.EDITABLE) ;
+	ATTR_CHARSET = AttributeRegistry.registerAttribute(cls,a);
+        // The content length attribute:
 	a = new IntegerAttribute("content-length"
 				 , null
 				 , Attribute.COMPUTED);
@@ -246,12 +258,12 @@ public class HTTPFrame extends ProtocolFrame {
     /**
      * The associated DirectoryResource (if any)
      */
-    protected DirectoryResource dresource = null;
+    protected DirectoryResource dresource  = null;
 
     /**
      * The associated FileResource (if any)
      */
-    protected FileResource      fresource = null;
+    protected FileResource      fresource  = null;
 
     /**
      * Register this frame to the given resource.
@@ -414,6 +426,8 @@ public class HTTPFrame extends ProtocolFrame {
 	super.setValue(idx, value);
 	if (idx == ATTR_CONTENT_TYPE)
 	    contenttype = null;
+	if (idx == ATTR_CHARSET)
+	    contenttype = null;
 	if (idx == ATTR_CONTENT_LENGTH)
 	    contentlength = null;
 	if ( idx == ATTR_CONTENT_ENCODING )
@@ -495,16 +509,47 @@ public class HTTPFrame extends ProtocolFrame {
 	return (String) getString (ATTR_CONTENT_ENCODING, def) ;
     }
 
+    /**
+     * Get this resource charset.
+     * @return A String, or <strong>null</strong> if not
+     *    defined.
+     */
+    public String getCharset() {
+	return (String) getValue(ATTR_CHARSET, null);
+    }
+
     /**
      * Get this resource content type.
      * @return An instance of MIMEType, or <strong>null</strong> if not
      *    defined.
      */
-
     public MimeType getContentType() {
 	return (MimeType) getValue(ATTR_CONTENT_TYPE, null);
     }
 
+    /**
+     * Get this resource Etag
+     * @return an instance of HttpEntityTag, or <strong>null</strong> if not
+     *    defined.
+     */
+
+    public HttpEntityTag getETag() {
+	if (etag == null) {
+	    String etag_s = null;
+	    if (fresource != null) {
+		long lstamp = fresource.getFileStamp();
+		if ( lstamp >= 0L ) {
+		    String soid  = Integer.toString(getOid(), 32);
+		    String stamp = Long.toString(lstamp, 32);
+		    etag_s = Integer.toString(getOid(), 32)+":"
+			+ Long.toString(lstamp, 32);
+		}
+	    }
+	    etag = HttpFactory.makeETag(false, etag_s);
+	}
+	return etag;
+    }
+
     /**
      * Get this resource content length.
      * @return The resource content length, or <strong>-1</strong> if not
@@ -734,9 +779,7 @@ public class HTTPFrame extends ProtocolFrame {
 	// Add style link
 	String css_url = getStyleSheetURL();
 	if (css_url != null) {
-	    g.addLink( new HtmlLink("STYLESHEET", 
-				    css_url, 
-				    MimeType.TEXT_CSS));
+	    g.addLink( new HtmlLink("STYLESHEET", css_url));
 	}
     }
 
@@ -805,13 +848,26 @@ public class HTTPFrame extends ProtocolFrame {
 		g.append("<p><a href=\"..\">Parent</a><br>");
 	    // List the children:
 	    for (int i = 0 ; i < resources.size() ; i++) {
-		String       name     = (String) resources.elementAt(i);
-		ResourceReference rr = null;
+		String            name = (String) resources.elementAt(i);
+		ResourceReference rr   = null;
+		long              size = -1;
 		rr = dresource.lookup(name);
 		FramedResource resource = null;
 		if (rr != null) {
 		    try {
 			resource = (FramedResource) rr.lock();
+			// remove manually deleted FileResources
+			if( resource instanceof FileResource ) {
+			    FileResource fr = (FileResource)resource;
+			    if( !fr.getFile().exists() ) {
+				try {
+				    fr.delete();
+				} catch (MultipleLockException ex) {};
+				continue;
+			    } else {
+				size = fr.getFile().length();
+			    }
+			}
 			HTTPFrame itsframe = null;
 			if (http_class != null)
 			    itsframe = 
@@ -824,23 +880,38 @@ public class HTTPFrame extends ProtocolFrame {
 					 getIconDirectory() +"/" + icon+
 					 "\">");
 			    // Resource's name with link:
-			    g.append("<a href=\"" 
-				     , resource.getURLPath()
-				     , "\">"+name+"</a>");
+			    if (resource instanceof ContainerInterface)
+				g.append("<a href=\"" 
+					 , URLEncoder.encode(name)
+					 , "/\">"+name+"</a>");
+			    else
+				g.append("<a href=\"" 
+					 , URLEncoder.encode(name)
+					 , "\">"+name+"</a>");
 			    // resource's title, if any:
 			    String title = itsframe.getTitle();
 			    if ( title != null )
 				g.append(" "+title);
-			    g.append("<br>");
+			    //size (if any)
+			    if (size != -1) {
+				String s = null;
+				if (size > 1023) {
+				    s = " ["+(size/1024)+" Kb]";
+				} else {
+				    s = " ["+size+" bytes]";
+				}
+				g.append(s);
+			    }
+			    g.append("<br>\n");
 			} else {
 			    // Resource's name with link:
 			    g.append(name+" (<i>Not available via HTTP.</i>)");
-			    g.append("<br>");
+			    g.append("<br>\n");
 			}
 		    } catch (InvalidResourceException ex) {
 			g.append(name+
 				 " cannot be loaded (server misconfigured)");
-			g.append("<br>");
+			g.append("<br>\n");
 			continue;
 		    } finally { 
 			rr.unlock();
@@ -879,8 +950,16 @@ public class HTTPFrame extends ProtocolFrame {
 
     protected void updateCachedHeaders() {
 	// Precompute a set of header values to keep by:
-	if ( contenttype == null )
-	    contenttype = HttpFactory.makeMimeType(getContentType());
+	if ( contenttype == null ) {
+	    String charset = getCharset();
+	    if (charset == null)
+		contenttype = HttpFactory.makeMimeType(getContentType());
+	    else {
+		MimeType ctype = getContentType().getClone();
+		ctype.addParameter("charset", charset);
+		contenttype = HttpFactory.makeMimeType(ctype);
+	    }
+	}
 	if (contentlength == null) {
 	    int cl = -1;
 	    if (fresource != null) 
@@ -901,14 +980,9 @@ public class HTTPFrame extends ProtocolFrame {
 	    contentlanguage = HttpFactory.makeStringList(getContentLanguage());
 
 	if (fresource != null) {
-	    // We only take car eof etag here:
+	    // We only take care of etag here:
 	    if ( etag == null ) {
-		long lstamp = fresource.getFileStamp();
-		if ( lstamp >= 0L ) {
-		    String soid  = Integer.toString(getOid(), 32);
-		    String stamp = Long.toString(lstamp, 32);
-		    etag = HttpFactory.makeETag(false, soid+":"+stamp);
-		}
+		getETag();
 	    }
 	    if (getMD5Flag() && (md5Digest == null)) {
 		getMd5Digest();
@@ -959,8 +1033,8 @@ public class HTTPFrame extends ProtocolFrame {
     
 	if (fresource != null) {
 	    // Set the entity tag:
-	    if ( etag != null )
-		reply.setHeaderValue(reply.H_ETAG, etag);
+	    if ( getETag() != null )
+		reply.setETag(etag);
 	    if ( acceptRanges )
 		reply.setHeaderValue(reply.H_ACCEPT_RANGES, _accept_ranges);
 	    if ( getMD5Flag()) {
@@ -1121,7 +1195,8 @@ public class HTTPFrame extends ProtocolFrame {
      * @param ls The current lookup state
      * @param lr The result
      * @return true if lookup is done.
-     * @exception ProtocolException If an error relative to the protocol occurs
+     * @exception ProtocolException If an error relative to the protocol
+     * occurs
      */
     protected boolean lookupDirectory(LookupState ls, LookupResult lr) 
 	throws ProtocolException
@@ -1198,6 +1273,8 @@ public class HTTPFrame extends ProtocolFrame {
 		    return true;
 		}
 	    } else if ( ! ls.isInternal() ) {
+		Request request = (Request)ls.getRequest() ;
+		request.setState(STATE_CONTENT_LOCATION, "true");
 		// return the index file.
 		String index = getIndex();
 		if ( index != null && index.length() > 0) {
@@ -1376,11 +1453,73 @@ public class HTTPFrame extends ProtocolFrame {
 	try { 
 	    reply.setStream(new FileInputStream(file));
 	} catch (IOException ex) {
-	    // I hate to have to loose time in tries
+	    Reply error = request.makeReply(HTTP.SERVICE_UNAVAILABLE);
+	    error.setContent(ex.getMessage());
+	    return error;
 	}
 	return reply ;
     }
 
+    /**
+     * Alway throws an HTTPException
+     */
+    protected Reply deleteMe(Request request) 
+	throws HTTPException
+    {
+	// Delete the resource if parent is extensible:
+	boolean extensible = false;
+	ResourceReference rr = fresource.getParent();
+	ResourceReference rrtemp = null;
+	Resource p = null;
+	while ( true ) {
+	    try {
+		if (rr == null)
+		    break;
+		p = rr.lock();
+		if (p instanceof DirectoryResource) {
+		    extensible = 
+			((DirectoryResource)p).getExtensibleFlag();
+		    break;
+		}
+		rrtemp = p.getParent();
+	    } catch (InvalidResourceException ex) {
+		break;
+	    } finally {
+		if (rr != null)
+		    rr.unlock();
+	    }
+	    rr = rrtemp;
+	}
+	if (extensible) {
+	    // The resource is indexed but has no file, emit an error
+	    String msg = fresource.getFile()+
+		": deleted, removing the FileResource.";
+	    getServer().errlog(fresource, msg);
+	    try {
+		fresource.delete();
+	    } catch (MultipleLockException ex) {
+		Reply error = request.makeReply(HTTP.NOT_FOUND) ;
+		error.setContentMD5(null); // FIXME must compute it!
+		error.setContent ("<h1>Document not found</h1>"+
+				  "<p>The document "+
+				  request.getURL()+
+				  " is indexed but not available."+
+				  "<p>"+ex.getMessage()+
+				  "<p>The server is misconfigured.") ;
+		throw new HTTPException (error) ;
+	    }
+	}
+	// Emit an error back:
+	Reply error = request.makeReply(HTTP.NOT_FOUND) ;
+	error.setContentMD5(null);
+	error.setContent ("<h1>Document not found</h1>"+
+			  "<p>The document "+
+			  request.getURL()+
+			  " is indexed but not available."+
+			  "<p>The server is misconfigured.") ;
+	return error;
+    }
+
     /**
      * Get for FileResource
      * @param request the incomming request.
@@ -1424,60 +1563,12 @@ public class HTTPFrame extends ProtocolFrame {
 	}
 	// Does this file really exists, if so send it back
 	if ( file.exists() ) {
-	    return createFileReply(request);
+	    reply = createFileReply(request);
+	    if (request.hasState(STATE_CONTENT_LOCATION))
+		reply.setContentLocation(getURL(request).toExternalForm());
+	    return reply;
 	} else {
-	    // Delete the resource if parent is extensible:
-	    boolean extensible = false;
-	    ResourceReference rr = fresource.getParent();
-	    ResourceReference rrtemp = null;
-	    Resource p = null;
-	    while ( true ) {
-		try {
-		    if (rr == null)
-			break;
-		    p = rr.lock();
-		    if (p instanceof DirectoryResource) {
-			extensible = 
-			    ((DirectoryResource)p).getExtensibleFlag();
-			break;
-		    }
-		    rrtemp = p.getParent();
-		} catch (InvalidResourceException ex) {
-		    break;
-		} finally {
-		    if (rr != null)
-			rr.unlock();
-		}
-		rr = rrtemp;
-	    }
-	    if (extensible) {
-		// The resource is indexed but has no file, emit an error
-		String msg = fresource.getFile()+
-		    ": deleted, removing the FileResource.";
-		getServer().errlog(fresource, msg);
-		try {
-		    fresource.delete();
-		} catch (MultipleLockException ex) {
-		    Reply error = request.makeReply(HTTP.NOT_FOUND) ;
-		    error.setContentMD5(null); // FIXME must compute it!
-		    error.setContent ("<h1>Document not found</h1>"+
-				      "<p>The document "+
-				      request.getURL()+
-				      " is indexed but not available."+
-				      "<p>"+ex.getMessage()+
-				      "<p>The server is misconfigured.") ;
-		    throw new HTTPException (error) ;
-		}
-	    }
-	    // Emit an error back:
-	    Reply error = request.makeReply(HTTP.NOT_FOUND) ;
-	    error.setContentMD5(null);
-	    error.setContent ("<h1>Document not found</h1>"+
-			      "<p>The document "+
-			      request.getURL()+
-			      " is indexed but not available."+
-			      "<p>The server is misconfigured.") ;
-	    throw new HTTPException (error) ;
+	    return deleteMe(request);
 	}
     }
 
@@ -1491,9 +1582,18 @@ public class HTTPFrame extends ProtocolFrame {
     protected Reply getDirectoryResource(Request request) 
 	throws ProtocolException, ResourceException
     {
+	String index = getIndex();
+	if ( index != null && index.length() > 0 ) {
+	    if (index.equals("*forbid*")) {
+		Reply rep = request.makeReply(HTTP.FORBIDDEN);
+		rep.setContent("<h1>Forbidden</h1>"+
+			       "The directory resource "+ request.getURL() +
+			       " cannot be browsed");
+		return rep;
+	   } 
+	}
 	return getDirectoryListing(request) ;
     }
-
     /**
      * The default HEAD method replies does a GET and removes entity.
      * @param request The request to handle.
@@ -1579,56 +1679,13 @@ public class HTTPFrame extends ProtocolFrame {
 	    return r;
 	}
 	if (! fresource.getFile().exists()) {
-	    // Delete the resource if parent is extensible:
-	    boolean extensible = false;
-	    ResourceReference rr = fresource.getParent();
-	    ResourceReference rrtemp = null;
-	    Resource p = null;
-	    while ( true ) {
-		try {
-		    if (rr == null)
-			break;
-		    p = rr.lock();
-		    if (p instanceof DirectoryResource)
-			extensible = 
-			    ((DirectoryResource)p).getExtensibleFlag();
-		    rrtemp = p.getParent();
-		} catch (InvalidResourceException ex) {
-		    break;
-		} finally {
-		    if (rr != null)
-			rr.unlock();
-		}
-		rr = rrtemp;
-	    }
-	    if (extensible) {
-		// The resource is indexed but has no file, emit an error
-		String msg = fresource.getFile()+
-		    ": deleted, removing the FileResource.";
-		getServer().errlog(fresource, msg);
-		try {
-		    fresource.delete();
-		} catch (MultipleLockException ex) {
-		    Reply error = request.makeReply(HTTP.NOT_FOUND) ;
-		    error.setContent ("<h1>Document not found</h1>"+
-				      "<p>The document "+
-				      request.getURL()+
-				      " is indexed but not available."+
-				      "<p>"+ex.getMessage()+
-				      "<p>The server is misconfigured.") ;
-		    throw new HTTPException (error) ;
-		}
-	    }
-	    // Emit an error back:
-	    Reply error = request.makeReply(HTTP.NOT_FOUND) ;
-	    error.setContent ("<h1>Document not found</h1>"+
-			      "<p>The document "+
-			      request.getURL()+
-			      " is indexed but not available."+
-			      "<p>The server is misconfigured.") ;
-	    throw new HTTPException (error) ;
+	    return deleteMe(request);
 	}
-	return createDefaultReply(request, HTTP.OK);
+	reply = createDefaultReply(request, HTTP.OK);
+	if (request.hasState(STATE_CONTENT_LOCATION))
+	    reply.setContentLocation(getURL(request).toExternalForm());
+	return reply;
+	
     }
 
     /**
@@ -1734,6 +1791,11 @@ public class HTTPFrame extends ProtocolFrame {
 	    if ( type == null ) {
 		setValue (ATTR_CONTENT_TYPE, rtype) ;
 	    } else if ( rtype.match (type) < 0 ) {
+		if (debug) {
+		    System.out.println("No match between: ["+
+				       rtype.toString()+"] and ["+
+				       type.toString()+"]");
+		}
 		Reply error = request.makeReply(HTTP.UNSUPPORTED_MEDIA_TYPE) ;
 		error.setContent ("<p>Invalid content type: "+type.toString());
 		throw new HTTPException (error) ;
@@ -1755,17 +1817,14 @@ public class HTTPFrame extends ProtocolFrame {
 	}
 	if ( status == HTTP.CREATED ) {
 	    reply = request.makeReply(status);
-	    // Henrik's fix, create the Etag on 201
+	    reply.setContent("<P>Resource succesfully created");
+	    if (request.hasState(STATE_CONTENT_LOCATION))
+		reply.setContentLocation(getURL(request).toExternalForm());
+            // Henrik's fix, create the Etag on 201
 	    if (fresource != null) {
 		// We only take car eof etag here:
 		if ( etag == null ) {
-		    long lstamp = fresource.getFileStamp();
-		    if ( lstamp >= 0L ) {
-			String soid  = Integer.toString(getOid(), 32);
-			String stamp = Long.toString(lstamp, 32);
-			etag = HttpFactory.makeETag(false, soid+":"+stamp);
-			reply.setHeaderValue(reply.H_ETAG, etag);
-		    }
+		    reply.setETag(getETag());
 		}
 	    }
 	    reply.setLocation(getURL(request));
@@ -1970,21 +2029,11 @@ public class HTTPFrame extends ProtocolFrame {
 	return reply;
     }
     
-    /**
-     * The handler for unknown method replies with a not implemented.
-     * @param request The request to handle.
-     * @exception ProtocolException Always thrown, to return a NOT_IMPLEMENTED
-     * error.
-     * @exception ResourceException If the resource got a fatal error.
-     */
-    public Reply extended(Request request)
-	throws ProtocolException, ResourceException
+    public Reply extended(Request request) 
+    	throws ProtocolException, ResourceException
     {
 	String method = request.getMethod() ;
-	if ((method != null) && method.equals("BROWSE") && getBrowsableFlag())
-	    return browse(request) ;
-
-	Reply error = request.makeReply(HTTP.NOT_IMPLEMENTED) ;
+	Reply error   = request.makeReply(HTTP.NOT_IMPLEMENTED) ;
 	error.setContent("Method "+method+" not implemented.") ;
 	throw new HTTPException (error) ;
     }
