@@ -1,5 +1,5 @@
 // Client.java
-// $Id: Client.java,v 1.52 1999/05/05 14:13:55 ylafon Exp $
+// $Id: Client.java,v 1.54.4.2 1999/11/30 15:51:31 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -69,7 +69,11 @@ class MimeClientFactory implements MimeParserFactory {
 
 public abstract class Client implements EventHandler { 
     private static final boolean debuglog = false;
-
+    private final static byte hexaTable[] = { 
+	(byte) '0', (byte) '1', (byte) '2', (byte) '3',
+	(byte) '4', (byte) '5', (byte) '6', (byte) '7',	 
+	(byte) '8', (byte) '9', (byte) 'A', (byte) 'B',
+	(byte) 'C', (byte) 'D', (byte) 'E', (byte) 'F' };
     /**
      * The continue reply, if ever needed is created only once.
      */
@@ -144,6 +148,10 @@ public abstract class Client implements EventHandler {
       * request 
       */
     protected boolean cont = false;
+    /**
+     * The number of bytes in the body of the previously handled request
+     */
+    protected long prev_body_count = 0;
 
     /**
      * Sets this client next timer. 
@@ -285,21 +293,31 @@ public abstract class Client implements EventHandler {
     protected int chunkTransfer(InputStream is)
 	throws IOException
     {
-	byte   crlf[]    = { ((byte) 13), ((byte) 10) } ;
-	byte   bheader[] = new byte[32] ;
-	int    blen      = 0 ;
-	int    written   = 0 ;
-	int    got       = 0 ;
-	String header    = null ;
+	byte   zeroChunk[] = { ((byte) 48), ((byte) 13), ((byte) 10) };//0\r\n
+	byte   crlf[]      = { ((byte) 13), ((byte) 10) } ; 
+	byte   bheader[]   = new byte[32] ;
+	int    blen        = 0 ;
+	int    written     = 0 ;
+	int    got         = 0 ;
+	int    sgot;
+	String header      = null ;
 
 	try {
 	    // Emit the reply stream:
 	    while ((got = is.read(buffer)) > 0) {
 		// Emit a full chunk: header first, followed by the body
-		header = Integer.toString(got, 16).toUpperCase() + "\r\n" ;
-		blen   = header.length() ;
-		header.getBytes(0, blen, bheader, 0) ;
-		output.write(bheader, 0, blen) ;
+		// we dump the hexa size of the header backward
+		sgot = got;
+		blen = 3;
+		bheader[30] = ((byte) 13); // \r
+		bheader[31] = ((byte) 10); // \n
+		while (sgot > 15) {
+		    bheader[32-blen] = hexaTable[sgot % 16];
+		    sgot >>= 4;
+		    blen++;
+		}
+		bheader[32-blen] = hexaTable[sgot];
+		output.write(bheader, 32-blen, blen) ;
 		output.write(buffer, 0, got) ;
 		output.write(crlf, 0, 2) ;
 		output.flush() ;
@@ -310,10 +328,8 @@ public abstract class Client implements EventHandler {
 	    // Anyway, if this really fails, the output will fail below too
 	}
 	// Emit the 0 chunk:
-	header = "0\r\n" ;
-	blen   = header.length() ;
-	header.getBytes(0, blen, bheader, 0) ;
-	output.write(bheader, 0, blen) ;
+	output.write(zeroChunk, 0, 3) ;
+	// FIXME trailers should be sent here
 	output.write(crlf, 0, 2) ;
 	output.flush() ;
 	return written + blen ;
@@ -439,6 +455,10 @@ public abstract class Client implements EventHandler {
 	    while ( (! interrupted)  && keep ) {
 		// Get the next available request, and  mark client as used:
 		try {
+		    // mark the stream, if some bytes are to be eaten
+		    if (prev_body_count > 0) {
+			input.mark(2048);
+		    }
 		    if ( processed == 0 ) {
 			// Always run for the first request, 
 			// update client's infos
@@ -480,6 +500,26 @@ public abstract class Client implements EventHandler {
 			    output.flush(); 
 		    }
 		    break alive_loop;
+		} catch (Exception genex) {
+		    if (debug)
+			genex.printStackTrace();
+		    // some bytes may have to be eaten
+		    // abomination, very ugly hack!
+		    if (genex.getMessage().startsWith("Bad request") && 
+			(prev_body_count > 0)) {
+			// check if the error is due to a
+			if (debug)
+			    System.out.println("Error after a body "+
+					       "request! (Skipping: " +
+					       prev_body_count+" bytes)");
+			// now try to eat some data...
+			input.reset();
+			byte b[] = new byte[(int)prev_body_count];
+			prev_body_count -= input.read(b);
+			    prev_body_count -= input.skip(prev_body_count);
+			    continue;
+		    }
+		    throw (genex);
 		}
 		// Some traces if required:
 		if ( debuglog )
@@ -502,6 +542,8 @@ public abstract class Client implements EventHandler {
 		    System.out.println(this
 				       + ", cl="+reply.getContentLength()
 				       + ", size="+sent);
+		// will be -1 if there is no body
+		prev_body_count = request.getContentLength();
 		log(request, reply, sent, tend-tstart) ;
 		processed++;
 		// If we can keep alive, and if the client doesn't do
@@ -524,7 +566,7 @@ public abstract class Client implements EventHandler {
 	    }
 	} catch (IOException ex) {
 	    if ( debug ) {
-		System.out.println("+++ "+this+" got exception:");
+		System.out.println("+++ "+this+" got IOException:");
 		ex.printStackTrace();
 	    }
 	    // Close any stream pending
@@ -539,7 +581,7 @@ public abstract class Client implements EventHandler {
 	    err = new ClientException(this, ex);
 	} catch (ClientException ex) {
 	    if ( debug ) {
-		System.out.println("+++ "+this+" got exception:");
+		System.out.println("+++ "+this+" got ClientException:");
 		ex.printStackTrace();
 	    }
 	    try {
