@@ -1,5 +1,5 @@
 // JigsawHttpServletRequest.java
-// $Id: JigsawHttpServletRequest.java,v 1.42 1999/03/17 18:00:56 bmahe Exp $
+// $Id: JigsawHttpServletRequest.java,v 1.46 1999/06/09 09:44:21 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -15,6 +15,7 @@ import org.w3c.util.*;
 import org.w3c.jigsaw.http.*;
 import org.w3c.jigsaw.forms.URLDecoder;
 import org.w3c.jigsaw.forms.URLDecoderException;
+import org.w3c.jigsaw.resources.VirtualHostResource;
 import org.w3c.www.http.*;
 import org.w3c.www.mime.*;
 import org.w3c.jigsaw.auth.AuthFilter; // for auth infos access
@@ -97,12 +98,43 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
 
     protected String requestedSessionID = null;
 
-    private synchronized void prepareQueryParameters()
-    {
+    private Hashtable mergeParameters(Hashtable source, Hashtable dest) {
+	if (source == null)
+	    return dest;
+	if (dest != null) {
+	    Enumeration e = source.keys();
+	    while (e.hasMoreElements()) {
+		String name = (String)e.nextElement();
+		Object value = dest.get(name);
+		if (value == null)
+		    dest.put(name, source.get(name));
+		else  if (value instanceof String[]) {
+		    String oldValues [] = (String[])value;
+		    String newValues [] = new String[oldValues.length+1];
+		    System.arraycopy(oldValues,0,newValues,0,
+				     oldValues.length);
+		    newValues[oldValues.length] = (String)source.get(name);
+		    dest.put(name,newValues);
+		} else {
+		    String newValues [] = new String[2];
+		    newValues[0] = (String)source.get(name);
+		    newValues[1] = (String)value;
+		    dest.put(name,newValues);
+		}
+	    }
+	    return dest;
+	} else {
+	    return source;
+	}
+    }
+
+    private synchronized void prepareQueryParameters() {
 	if( queryParameters != null )
 	    return;
+	Hashtable postParameters = null;
 	// What kinf of parameters are we expecting ?
 	if ( request.getMethod().equals("POST") ) {
+	    postParameters = new Hashtable(2);
 	    // POSTed parameters, check content type:
 	    if ((! request.hasContentType())
 		|| (type.match(request.getContentType()) < 0) ) 
@@ -116,52 +148,27 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
 		if ( client != null ) 
 		    client.sendContinue();
 		dec = new URLDecoder (in, false);
-		queryParameters = dec.parse () ;
+		postParameters = dec.parse () ;
 	    } catch (URLDecoderException e) {
-		queryParameters = null;
+		postParameters = null;
 	    } catch (IOException ex) {
-		queryParameters = null;
-	    }
-	} else {
-	    // URL encoded parameters:
-	    String query = request.getQueryString();
-	    if (query != null) {
-		Reader qis = new StringReader(query);
-		try {
-		    queryParameters = new URLDecoder(qis, false).parse();
-		} catch (Exception ex) {
-		    throw new RuntimeException("Java implementation bug.");
-		}
+		postParameters = null;
 	    }
 	}
+	// URL encoded parameters:
+	String query = request.getQueryString();
+	if (query != null) {
+	    Reader qis = new StringReader(query);
+	    try {
+		queryParameters = new URLDecoder(qis, false).parse();
+	    } catch (Exception ex) {
+		throw new RuntimeException("Java implementation bug.");
+	    }
+	}
+	queryParameters = mergeParameters(postParameters, queryParameters);
 	// state parameters
 	Hashtable param = (Hashtable)request.getState(STATE_PARAMETERS);
-	if (param != null) {
-	    if (queryParameters == null)
-		queryParameters = param;
-	    else {
-		Enumeration e= param.keys();
-		while (e.hasMoreElements()) {
-		    String name = (String)e.nextElement();
-		    Object value = queryParameters.get(name);
-		    if (value == null)
-			queryParameters.put(name,param.get(name));
-		    else  if (value instanceof String[]) {
-			String oldValues [] = (String[])value;
-			String newValues [] = new String[oldValues.length+1];
-			System.arraycopy(oldValues,0,newValues,0,
-					 oldValues.length);
-			newValues[oldValues.length] = (String)param.get(name);
-			queryParameters.put(name,newValues);
-		    } else {
-			String newValues [] = new String[2];
-			newValues[0] = (String)param.get(name);
-			newValues[1] = (String)value;
-			queryParameters.put(name,newValues);
-		    }
-		}
-	    }
-	}
+	queryParameters = mergeParameters(param, queryParameters);
     }
 
     protected String getURLParameter(String name) {
@@ -280,12 +287,51 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * ServletRequest implementation - Get real path.
      * Jigsaw realy has no notion of <em>translation</em> stricto
      * sensu (it has much better in fact ;-). This is a pain here.
-     * @return Always <strong>null</strong>.
+     * @return the real path.
      * @deprecated since jsdk1.2
      */
 
     public String getRealPath(String name) {
-	return null;
+	httpd             server  = request.getClient().getServer();
+	ResourceReference rr_root = server.getRootReference();
+	try {
+	    FramedResource root = (FramedResource)rr_root.lock();
+	    if (root instanceof VirtualHostResource) {
+		//backward to the virtual host resource
+		ResourceReference rr  = request.getTargetResource();
+		ResourceReference rrp = null;
+		while (true) {
+		    try {
+			FramedResource res = (FramedResource)rr.lock();
+			rrp = res.getParent();
+			if ((rrp == rr_root) || (rrp == null)) {
+			    if (res.definesAttribute("directory")) {
+				File dir = 
+				    (File)res.getValue("directory", null);
+				if (dir != null)
+				    return (new File(dir, name)).
+					getAbsolutePath();
+			    }
+			    return null;
+			}
+		    } catch (InvalidResourceException ex) {
+			return null;
+		    } finally {
+			rr.unlock();
+		    }
+		    rr = rrp;
+		}
+	    } else if (root.definesAttribute("directory")) {
+		File dir = (File)root.getValue("directory", null);
+		if (dir != null)
+		    return (new File(dir, name)).getAbsolutePath();
+	    }
+	    return null;
+	} catch (InvalidResourceException ex) {
+	    return null;
+	} finally {
+	    rr_root.unlock();
+	}
     }
 
     protected ServletInputStream is = null;
@@ -421,44 +467,8 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
     public  String getPathTranslated()
     {
 	String pathinfo = getPathInfo();
-	if ( pathinfo != null ) {
-	    httpd             server  = request.getClient().getServer();
-	    ResourceReference rr_root = server.getRootReference();
-	    try {
-		LookupState       ls      = new LookupState(pathinfo);
-		LookupResult      lr      = new LookupResult(rr_root);
-		ResourceReference path    = null;
-		
-		try {
-		    FramedResource root = (FramedResource) rr_root.lock();
-		    if (root.lookup(ls,lr))
-			path = lr.getTarget();
-		} catch (InvalidResourceException ex) {
-		    path = null;
-		} finally {
-		    rr_root.unlock();
-		}
-		
-		if (path != null) {
-		    try {
-			Resource r = path.lock();
-			if (r instanceof FileResource)
-			    return ((FileResource)
-				    r).getFile().getAbsolutePath();
-			else if (r instanceof DirectoryResource)
-			    return ((DirectoryResource)
-				    r).getDirectory().getAbsolutePath();
-			else return null;
-		    } catch (InvalidResourceException ex) {
-			return null;
-		    } finally {
-			path.unlock();
-		    }
-		}
-	    } catch (org.w3c.tools.resources.ProtocolException ex) {
-		return null;
-	    }
-	}
+	if ( pathinfo != null )
+	    return getRealPath(pathinfo);
 	return null;
     }
 
@@ -799,8 +809,26 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
 	    throw new IllegalStateException("Input Stream used");
 	stream_state = STREAM_READER_USED;
 	if (reader == null) {
-	    reader = new BufferedReader(
-			new InputStreamReader( getJigsawInputStream()));
+	    String enc[] = request.getContentEncoding();
+	    InputStream is = getJigsawInputStream();
+	    if (enc != null) {
+		InputStreamReader isr = null;
+		for (int i=0; i < enc.length; i++) {
+		    try {
+			isr = new InputStreamReader(is, enc[i]);
+			break;
+		    } catch (UnsupportedEncodingException ex) {
+			// not a valid encoding, skip it
+			isr = null;
+		    }
+		}
+		if (isr != null)
+		    reader = new BufferedReader(isr);
+		else
+		    reader = new BufferedReader(new InputStreamReader(is));
+	    } else {
+		reader = new BufferedReader(new InputStreamReader(is));	
+	    }
 	}
 	return reader;
     }
