@@ -1,5 +1,5 @@
 // JigsawServletContext.java
-// $Id: JigsawServletContext.java,v 1.19 1999/01/11 10:58:37 bmahe Exp $
+// $Id: JigsawServletContext.java,v 1.24 1999/08/09 08:39:45 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1998.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -16,9 +16,11 @@ import org.w3c.tools.resources.store.*;
 import org.w3c.tools.resources.event.*;
 import org.w3c.tools.resources.*;
 import org.w3c.jigsaw.frames.*;
+import org.w3c.jigsaw.resources.VirtualHostResource;
+import org.w3c.jigsaw.http.httpd;
 
 /**
- * @version $Revision: 1.19 $
+ * @version $Revision: 1.24 $
  * @author  Benot Mah (bmahe@w3.org)
  */
 public class JigsawServletContext extends StructureChangedAdapter
@@ -79,6 +81,15 @@ public class JigsawServletContext extends StructureChangedAdapter
 	    log(msg+" : "+ex.getClass().getName()+" ("+ex.getMessage()+")");
 	}
 
+	void log(Throwable throwable, String msg) {
+	    ByteArrayOutputStream out = new ByteArrayOutputStream();
+	    PrintWriter writer = new PrintWriter(out);
+	    throwable.printStackTrace(writer);
+	    writer.close();
+	    String stacktrace = out.toString();
+	    log(msg+" "+stacktrace);
+	}
+
 	void openLogFile() 
 	    throws IOException
 	{
@@ -115,6 +126,8 @@ public class JigsawServletContext extends StructureChangedAdapter
 
     private File directory = null;
 
+    private Hashtable attributes = null;
+
     protected static String logdir     = "logs" ;
 
     protected static String deflogfile = "servlets";
@@ -140,6 +153,10 @@ public class JigsawServletContext extends StructureChangedAdapter
 	}
     }
 
+    protected long getServletTimeout() {
+	return props.getLong(ServletProps.SERVLET_TIMEOUT, -1);
+    }
+
     /**
      * A useful utility routine that tries to guess the content-type
      * of an object based upon its extension.
@@ -193,6 +210,7 @@ public class JigsawServletContext extends StructureChangedAdapter
 
     /**
      * ServletContext implementation - Lookup a given servlet.
+     * @deprecated since jsdk2.1
      */
 
     public Servlet getServlet(String name) {
@@ -209,6 +227,7 @@ public class JigsawServletContext extends StructureChangedAdapter
 
     /**
      * ServletContext implementation - Enumerate all servlets within context.
+     * @deprecated since jsdk2.1
      */
 
     public Enumeration getServlets() {
@@ -226,6 +245,7 @@ public class JigsawServletContext extends StructureChangedAdapter
     /**
      * ServletContext implementation - Enumerate all servlets names 
      * within context.
+     * @deprecated since jsdk2.1
      */
 
     public Enumeration getServletNames() {
@@ -248,16 +268,73 @@ public class JigsawServletContext extends StructureChangedAdapter
 	logger.log(msg);
     }
     
+    /**
+     * @deprecated since jsdk2.1
+     */
     public void log(Exception ex, String msg) {
 	logger.log(ex,msg);
     }
 
+    public void log(String message, Throwable throwable) {
+	logger.log(throwable, message);
+    }
+
     /**
      * ServletContext implementation - Translate a piece of path.
      */
 
     public String getRealPath(String path) {
-	return path;
+	httpd             server  = (httpd)getServer();
+	ResourceReference rr_root = server.getRootReference();
+	try {
+	    FramedResource root = (FramedResource)rr_root.lock();
+	    if (root instanceof VirtualHostResource) {
+		//backward to the virtual host resource
+		ResourceReference rr  = null;
+		ResourceReference rrp = null;
+		FramedResource    res = null;
+		try {
+		    res = (FramedResource)reference.lock();
+		    ServletDirectoryFrame fr = (ServletDirectoryFrame)res;
+		    rr = fr.getResource().getResourceReference();
+		} catch (InvalidResourceException ex) {
+		    return null;
+		} finally {
+		    reference.unlock();
+		}
+
+		while (true) {
+		    try {
+			res = (FramedResource)rr.lock();
+			rrp = res.getParent();
+			if ((rrp == rr_root) || (rrp == null)) {
+			    if (res.definesAttribute("directory")) {
+				File dir = 
+				    (File)res.getValue("directory", null);
+				if (dir != null)
+				    return (new File(dir, path)).
+					getAbsolutePath();
+			    }
+			    return null;
+			}
+		    } catch (InvalidResourceException ex) {
+			return null;
+		    } finally {
+			rr.unlock();
+		    }
+		    rr = rrp;
+		}
+	    } else if (root.definesAttribute("directory")) {
+		File dir = (File)root.getValue("directory", null);
+		if (dir != null)
+		    return (new File(dir, path)).getAbsolutePath();
+	    }
+	    return null;
+	} catch (InvalidResourceException ex) {
+	    return null;
+	} finally {
+	    rr_root.unlock();
+	}
     }
 
     /**
@@ -284,17 +361,34 @@ public class JigsawServletContext extends StructureChangedAdapter
      */
     
     public Object getAttribute(String name) {
-	try {
-	    Resource res = reference.lock();
-	    return ((ServletDirectoryFrame)res).getAttribute(name);
-	} catch(InvalidResourceException ex) {
-	    ex.printStackTrace();
-	    return null;
-	} finally {
-	    reference.unlock();
+	Object attribute = attributes.get(name);
+	if (attribute != null) {
+	    return attribute;
+	} else {
+	    try {
+		Resource res = reference.lock();
+		return ((ServletDirectoryFrame)res).getAttribute(name);
+	    } catch(InvalidResourceException ex) {
+		ex.printStackTrace();
+		return null;
+	    } finally {
+		reference.unlock();
+	    }
 	}
     }
 
+    public void setAttribute(String name, Object object) {
+	attributes.put(name, object);
+    }
+
+    public void removeAttribute(String name) {
+	attributes.remove(name);
+    }
+
+    public Enumeration getAttributeNames() {
+	return attributes.keys();
+    }
+
     private AutoReloadServletLoader loader = null;
 
     /** 
@@ -323,6 +417,120 @@ public class JigsawServletContext extends StructureChangedAdapter
 	return directory;
     }
 
+    //jsdk2.1
+
+    /**
+     * Returns a RequestDispatcher object for the specified URL path if 
+     * the context knows of an active source (such as a servlet, JSP page,
+     * CGI script, etc) of content for the particular path. This format of
+     * the URL path must be of the form /dir/dir/file.ext. The servlet 
+     * engine is responsible for implementing whatever functionality is 
+     * required to wrap the target source with an implementation of
+     * the RequestDispatcher interface. 
+     * @param urlpath Path to use to look up the target server resource
+     */
+    public RequestDispatcher getRequestDispatcher(String urlpath) {
+	return 
+	    JigsawRequestDispatcher.getRequestDispatcher(urlpath, 
+							 (httpd)getServer());
+    }
+
+    public int getMajorVersion() {
+	return 2;
+    }
+
+    public int getMinorVersion() {
+	return 1;
+    }
+    
+    public ServletContext getContext(String uripath) {
+	if (uripath == null)
+	    return null;
+	//first, find the ServletDirectoryFrame.
+	// Prepare for lookup:
+	ResourceReference rr_root = null;
+	rr_root = ((httpd) getServer()).getRootReference();
+
+	FramedResource root = null;
+	root = ((httpd) getServer()).getRoot();
+
+	// Do the lookup:
+	ResourceReference r_target = null;
+	try {
+	    LookupState  ls = new LookupState(uripath);
+	    LookupResult lr = new LookupResult(rr_root);
+	    root.lookup(ls, lr);
+	    r_target = lr.getTarget();
+	} catch (Exception ex) {
+	    r_target = null;
+	}
+	//then return its context
+	if (r_target != null) {
+	    try {
+		Resource target = r_target.lock();
+		if (target instanceof FramedResource) {
+		    ServletDirectoryFrame frame = (ServletDirectoryFrame)
+			((FramedResource) target).
+		      getFrame("org.w3c.jigsaw.servlet.ServletDirectoryFrame");
+		    if (frame != null)
+			return frame.getServletContext();
+		}
+	    } catch (InvalidResourceException ex) {
+		// continue
+	    } finally {
+		r_target.unlock();
+	    }
+	}
+	return null;
+    }
+
+    public URL getResource(String path) 
+        throws MalformedURLException
+    {
+        if (path.length() > 0 && path.charAt(0) == '/')
+            path = path.substring(1);
+        if (File.separatorChar != '/')
+            path.replace('/', File.separatorChar);
+        String dir = getServletDirectory().getAbsolutePath();
+        File file = new File(dir, path);
+        if (! file.exists())
+            return null;
+        String url = file.getAbsolutePath();
+        if (File.separatorChar != '/')
+            url = url.replace(File.separatorChar, '/');
+        if (url.length() > 0 && url.charAt(0) != '/')
+            url = '/' + url;
+        return new URL("file", "", url);
+    }
+
+//     public URL getResource(String path) 
+// 	throws MalformedURLException
+//     {
+// 	String url = getServletDirectory().getAbsolutePath();
+// 	if ( File.separatorChar != '/' )
+// 	    url = url.replace(File.separatorChar, '/');
+// 	File file = new File(url,path);
+// 	if (! file.exists())
+// 	    return null;
+// 	return new URL("file", "localhost", url+"/"+path);
+//     }
+
+    public InputStream getResourceAsStream(String path) {
+	try {
+	    URL resource = getResource(path);
+	    if (resource == null)
+		return null;
+	    try {
+		URLConnection c = resource.openConnection();
+		return c.getInputStream();
+	    } catch (IOException ex) {
+		return null;
+	    }
+	} catch (MalformedURLException ex) {
+	    return null;
+	}
+    }
+
     /**
      * Create a new ServletContext.
      * @param ref a ResourceReference pointing on a ServletDirectoryFrame.
@@ -330,11 +538,14 @@ public class JigsawServletContext extends StructureChangedAdapter
     protected JigsawServletContext(ResourceReference ref, 
 				   ObservableProperties props) 
     {
-	this.reference = ref;
-	this.props     = props;
+	this.reference  = ref;
+	this.props      = props;
+	this.attributes = new Hashtable(3);
+	this.logger     = new Logger(getServletLogFile());
+	this.loader     = new AutoReloadServletLoader(this);
+
 	props.registerObserver(this);
-	this.logger = new Logger(getServletLogFile());
-	this.loader = new AutoReloadServletLoader(this);
+
 	try {
 	    Resource res = reference.lock();
 	    if (! (res instanceof ServletDirectoryFrame)) {
