@@ -1,5 +1,5 @@
 // FramedResource.java
-// $Id: FramedResource.java,v 1.22 1999/04/28 08:05:30 bmahe Exp $
+// $Id: FramedResource.java,v 1.22.4.2 2000/01/20 16:18:49 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -573,6 +573,29 @@ public class FramedResource extends Resource
 	}
     }
 
+    /**
+     * Set a value, without posting event.
+     * @param idx The attribute index, in the list of attributes advertized by 
+     * the resource.
+     * @param value The new value for this attribute.
+     */
+    public synchronized void setSilentValue(int idx, Object value) {
+	disableEvent();
+	super.setValue(idx, value);
+	enableEvent();
+    }
+
+    /**
+     * Set a value, without posting event.
+     * @param name The attribute name.
+     * @param value The new value for the attribute.
+     */
+    public synchronized void setSilentValue(String name, Object value) {
+	disableEvent();
+	super.setValue(name, value);
+	enableEvent();
+    }
+
     /**
      * Lookup the target resource.
      * @param ls The current lookup state
@@ -654,7 +677,7 @@ public class FramedResource extends Resource
 	// Initialize the frames if any.
 	ResourceFrame frames[] = getFrames();
 	if ( frames != null ) {
-	    this.framesRef = new Hashtable(frames.length);
+	    this.framesRef = new Hashtable(Math.max(frames.length, 1));
 	    Hashtable defs = new Hashtable(3);
 	    for (int i = 0 ; i < frames.length ; i++) {
 		if ( frames[i] == null )
