@@ -1,5 +1,5 @@
 // HttpMessage.java
-// $Id: HttpMessage.java,v 1.36 1999/06/02 11:57:50 bmahe Exp $
+// $Id: HttpMessage.java,v 1.36.4.2 1999/12/02 18:06:14 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -664,11 +664,18 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
 		// ugly hack :(
 		HttpSetCookieList hscl = (HttpSetCookieList) v;
 		int nbc = hscl.length();
-		for (int j = 0 ; j < nbc ; j++) {
+		if (nbc == 0) { //the ugliest hack I have ever wrote ;-)
 		    out.write(d.getTitle());
 		    out.write(':'); out.write(' ');
-		    hscl.emitCookie(out, j);
+		    v.emit(out);
 		    out.write('\r'); out.write('\n');
+		} else {
+		    for (int j = 0 ; j < nbc ; j++) {
+			out.write(d.getTitle());
+			out.write(':'); out.write(' ');
+			hscl.emitCookie(out, j);
+			out.write('\r'); out.write('\n');
+		    }
 		}
 	    } else if ( v != null ) {
 		out.write(d.getTitle());
@@ -787,6 +794,31 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
 	return (cc == null) ? -1 : cc.getMaxAge();
     }
 
+    /**
+     * Set the <code>s-maxage</code> value of the associated cache control.
+     * Set the s-maxage value on an HTTP/1.1 reply
+     * @param maxage The max-age value in seconds, or <strong>-1</strong>
+     * to reset the value.
+     */
+
+    public void setSMaxAge(int smaxage) {
+	HttpCacheControl cc = getCacheControl();
+	if ( cc == null ) {
+	    setCacheControl(cc = new HttpCacheControl(true));
+	}
+	cc.setSMaxAge(smaxage);
+    }
+
+    /**
+     * Get the <code>s-maxage</code> value for the current cache control.
+     * @return The s-maxage value, as an integer (seconds), or 
+     * <strong>-1</strong> if undefined.
+     */
+    public int getSMaxAge() {
+	HttpCacheControl cc = getCacheControl();
+	return (cc == null) ? -1 : cc.getSMaxAge();
+    }
+
     /**
      * Get the <code>no-cache</code> directive of the cache control header.
      * @return A list of token (potentially empty) encoded as an array of
