@@ -1,5 +1,5 @@
 // HttpBasicServer.java
-// $Id: HttpBasicServer.java,v 1.31 1999/01/19 16:10:42 ylafon Exp $
+// $Id: HttpBasicServer.java,v 1.31.4.1 1999/11/30 10:30:22 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -505,12 +505,19 @@ public class HttpBasicServer extends HttpServer {
 	    if (reply.getStatus() == HTTP.EXPECTATION_FAILED)
 		return reply; // FIXME observer?
 	    // Notify the observer if any:
-	    if ( o != null ) 
+	    if ( o != null ) {
 		notifyObserver(o, new ContinueEvent(this, request, reply));
+	    }
 	    // Finish the request normally:
 	    request.emit(os, Request.EMIT_BODY|Request.EMIT_FOOTERS);
 	    os.flush();
 	    reply = (Reply) p.parse();
+	    // if we don't have any observer, we should eat the 100 continue!
+	    if ( o == null) {
+		while ((reply.getStatus() / 100) == 1 ) {
+		    reply = (Reply) p.parse();
+		}
+	    }
 	}
 	return reply;
     }
