@@ -1,5 +1,5 @@
 // HttpManager.java
-// $Id: HttpManager.java,v 1.44 1998/12/23 22:00:58 ylafon Exp $
+// $Id: HttpManager.java,v 1.52.4.5 2000/01/21 10:16:03 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -21,12 +21,21 @@ class ManagerDescription {
     }
 
     final boolean sameProperties(Properties props) {
-	return props == properties;
+	if (props.size() != properties.size())
+	    return false;
+	Enumeration enum = props.propertyNames();
+	while (enum.hasMoreElements()) {
+	    String name = (String) enum.nextElement();
+	    String prop = properties.getProperty(name);
+	    if ((prop == null) || (! prop.equals(props.getProperty(name))))
+		return false;
+	}
+	return true;
     }
 
     ManagerDescription(HttpManager manager, Properties props) {
 	this.manager    = manager;
-	this.properties = props;
+	this.properties = (Properties)props.clone();
     }
 }
 
@@ -56,6 +65,9 @@ class ReplyFactory implements MimeParserFactory {
  */
 
 public class HttpManager implements PropertyMonitoring {
+
+    private static final boolean debug = false;
+
     private static final 
     String DEFAULT_SERVER_CLASS = "org.w3c.www.protocol.http.HttpBasicServer";
 
@@ -141,7 +153,7 @@ public class HttpManager implements PropertyMonitoring {
      * The default value for the <code>User-Agent</code> header.
      */
     public static final
-    String DEFAULT_USER_AGENT = "Jigsaw/2.0.0";
+    String DEFAULT_USER_AGENT = "Jigsaw/2.0.4";
 
     /**
      * This array keeps track of all the created managers.
@@ -198,7 +210,7 @@ public class HttpManager implements PropertyMonitoring {
 		return false;
 	    }
 	    // Now if a proxy...
-	    if ( proxy != null )
+	    if (( proxy != null ) && (proxy.getHost() != null))
 		template.setProxy(proxy);
 	} else {
 	    template.setProxy(null);
@@ -298,9 +310,8 @@ public class HttpManager implements PropertyMonitoring {
 	for (int i = 0 ; i < managers.length ; i++) {
 	    if ( managers[i] == null )
 		continue;
-	    if ( managers[i].sameProperties(p) ) {
+	    if ( managers[i].sameProperties(p) )
 		return managers[i].getManager();
-	    }
 	}
 	// Get the props we will initialize from:
 	ObservableProperties props = null;
@@ -462,6 +473,8 @@ public class HttpManager implements PropertyMonitoring {
      */
 
     public void notifyUse(HttpConnection conn) {
+	if (debug)
+	    System.out.println("+++ connection used");
 	connectionsLru.remove(conn);
     }
 
@@ -470,10 +483,13 @@ public class HttpManager implements PropertyMonitoring {
      * @param conn The connection that is now idle.
      */
 
-    public void notifyIdle(HttpConnection conn) {
+    public synchronized void notifyIdle(HttpConnection conn) {
+	if (debug)
+	    System.out.println("+++ connection idle");
 	connectionsLru.toHead(conn);
 	HttpServerState ss = conn.getServer().getState();
 	ss.registerConnection(conn);
+	notify();
     }
 
     /**
@@ -482,6 +498,9 @@ public class HttpManager implements PropertyMonitoring {
      */
 
     protected synchronized void notifyConnection(HttpConnection conn) {
+	if (debug)
+	    System.out.println("+++ notify conn_count " + (conn_count+1)
+		               + " / " + conn_max);
 	if ( ++conn_count > conn_max )
 	    closeAnyConnection();
     }
@@ -496,12 +515,14 @@ public class HttpManager implements PropertyMonitoring {
 	ss.deleteConnection(conn);
 	synchronized(this) {
 	    --conn_count;
-	    notifyAll();
+	    if (debug)
+		System.out.println("+++ delete conn_count: " + conn_count);
+	    notify();
 	}
     }
 
     protected synchronized boolean tooManyConnections() {
-	return conn_count > conn_max;
+	return conn_count >= conn_max;
     }
 
     /**
@@ -515,6 +536,12 @@ public class HttpManager implements PropertyMonitoring {
 	return ss.getConnection();
     }
 
+    /**
+     * Wait for a connection to come up.
+     * @param server, the target server.
+     * @exception InterruptedException If interrupted..
+     */
+
     protected synchronized void waitForConnection(HttpServer server)
 	throws InterruptedException
     {
@@ -522,20 +549,29 @@ public class HttpManager implements PropertyMonitoring {
     }
 
     /**
-     * Close one connection, but pickling the least recently used one.
+     * Close some connections, but pickling the least recently used ones.
+     * One third of the max number of connection is cut. This is done to 
+     * eliminate the old connections that should be broken already.
+     * (no Socket.isAlive());
      * @return A boolean, <strong>true</strong> if a connection was closed
      * <strong>false</strong> otherwise.
      */
 
     protected boolean closeAnyConnection() {
-	HttpConnection conn = (HttpConnection) connectionsLru.removeTail();
-	if ( conn != null ) {
-	    conn.close();
-	    deleteConnection(conn);
-	    return true;
-	} else {
-	    return false;
+	boolean saved = false;
+	int max = Math.max(conn_max/3, 1);
+	for (int i=0; i < max; i++) {
+	    HttpConnection conn = (HttpConnection) connectionsLru.removeTail();
+	    if ( conn != null ) {
+		conn.close();
+		deleteConnection(conn);
+		if (debug)
+		    System.out.println("+++ close request");
+		    saved = true;
+	    } else
+		break;
 	}
+	return saved;
     }
 
     /**
@@ -569,6 +605,8 @@ public class HttpManager implements PropertyMonitoring {
     protected final synchronized void incrConnCount(HttpServer server) {
 	if ( ++conn_count > conn_max )
 	    closeAnyConnection();
+	if (debug)
+	    System.out.println("+++ incr conn_count: " + conn_count);
     }
 
     /**
@@ -578,6 +616,8 @@ public class HttpManager implements PropertyMonitoring {
 
     protected final synchronized void decrConnCount(HttpServer server) {
 	--conn_count;
+	if (debug)
+	    System.out.println("+++ decr conn_count: " + conn_count);
     }
 
     /**
@@ -844,9 +884,9 @@ public class HttpManager implements PropertyMonitoring {
 	    PropRequestFilter filter = 
 	      new org.w3c.www.protocol.http.cookies.CookieFilter();
 	    filter.initialize(manager);
-	    PropRequestFilter debug = 
+	    PropRequestFilter pdebug = 
 	      new org.w3c.www.protocol.http.DebugFilter();
-	    debug.initialize(manager);
+	    pdebug.initialize(manager);
 	    Request request = manager.createRequest();
 	    request.setURL(new URL(args[0]));
 	    request.setMethod("GET");
@@ -859,7 +899,7 @@ public class HttpManager implements PropertyMonitoring {
 	    byte buf[] = new byte[4096];
 	    int  cnt   = 0;
 	    while ((cnt = in.read(buf)) > 0) 
-	      System.out.print(new String(buf, 0, 0, cnt));
+	      System.out.print(new String(buf, 0, cnt));
 	    System.out.println("-");
 	    in.close();
 	    manager.sync();
