@@ -1,5 +1,5 @@
 // CacheFilter.java
-// $Id: CacheFilter.java,v 1.49 1998/10/23 13:26:07 ylafon Exp $
+// $Id: CacheFilter.java,v 1.51.2.3 1999/11/25 13:41:34 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -16,6 +16,7 @@ import org.w3c.tools.resources.*;
 import org.w3c.tools.resources.store.*;
 import org.w3c.www.protocol.http.*;
 import org.w3c.www.http.*;
+import org.w3c.www.mime.*;
 
 public class CacheFilter implements PropRequestFilter, PropertyMonitoring {
     /**
@@ -600,6 +601,18 @@ public class CacheFilter implements PropRequestFilter, PropertyMonitoring {
 	int len = reply.getContentLength();
 	if ( (len >= size*fileSizeRatio) && garbageCollectionEnabled )
 	    return false;
+	// check if it is a multipart with no size 
+	// (ugly hack for server push)
+	if (len == -1) {
+	    try {
+		MimeType rep_mt = reply.getContentType();
+		if (rep_mt != null)
+		    if (rep_mt.match(new MimeType("multipart/*")) > 0)
+			return false;
+	    } catch (Exception ex) {
+		// MimeTypeFormatException should never be fired here
+	    }
+	}
 	// Check the status code:
 	switch(reply.getStatus()) {
 	  case HTTP.OK:
@@ -662,6 +675,9 @@ public class CacheFilter implements PropRequestFilter, PropertyMonitoring {
 	}
 	if ( reply.hasPragma("no-cache") )
 	    return false;
+	// ugly hack for bad sites with cookies
+	if (reply.getSetCookie() != null)
+	    return false;
 	// Yeah !
 	return true;
     }
@@ -698,7 +714,6 @@ public class CacheFilter implements PropRequestFilter, PropertyMonitoring {
 				 + ". <p>It is disconnected.");
 		return reply;
 	    } else {
-		// If this is a PUT, invalid relevant cache entry:
 		if ( debug )
 		    trace(request, "can't use cache.");
 		// can't use cache, then we must invalidate the entry!
@@ -707,8 +722,15 @@ public class CacheFilter implements PropRequestFilter, PropertyMonitoring {
 		// or the one used for the request
 		// for now on, use only the on of the client 
 		// ir invalidate cache entry
-//		if ( request.getMethod().equals("PUT") ) 
-		    store.removeResource(request.getURL().toExternalForm());
+		CachedResource ires = null;
+		try {
+		    ires = loadResource(request.getURL().toExternalForm());
+		} catch (InvalidResourceException ex) {
+		    ires = null;
+		}
+		if (ires != null) {
+		    ires.setWillRevalidate(true);
+		}
 		request.setState(STATE_NOCACHE, Boolean.TRUE);
 		return null;
 	    }
@@ -744,6 +766,8 @@ public class CacheFilter implements PropRequestFilter, PropertyMonitoring {
 		// We really want to use the cache, whatever
 		addWarning(request, WARN_DISCONNECTED);
 	    } else {
+		if (debug)
+		    System.out.println("Needs revalidation");
 		// Let's continue through the revalidation process:
 		request.setState(STATE_REVALIDATION, Boolean.TRUE);
 		return null;
@@ -775,7 +799,11 @@ public class CacheFilter implements PropRequestFilter, PropertyMonitoring {
 				 , ((reply.getStatus() == HTTP.NOT_MODIFIED)
 				    ? HOW_REVALIDATION_SUCCESS
 				    : HOW_REVALIDATION_FAILURE));
-		return c.validate(request, reply);
+		Reply valrep = c.validate(request, reply);
+		// Client or Server error, ask for revalidation
+		if (valrep.getStatus()/100 >= 4) {
+		    c.setWillRevalidate(true);
+		}
 	    } catch (IOException ex) {
 		if ( debug )
 		    trace(request, "cache IO error.");
