@@ -1,5 +1,5 @@
 // CachedResource.java
-// $Id: CachedResource.java,v 1.50 1998/09/01 14:50:22 ylafon Exp $
+// $Id: CachedResource.java,v 1.52.2.4 1999/11/26 17:12:32 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -19,6 +19,8 @@ import org.w3c.www.mime.*;
 import org.w3c.jigsaw.frames.MimeTypeAttribute;
 
 public class CachedResource extends Resource implements TeeMonitor {
+    private static final boolean debug = false;
+
     private static final int ACTIVE_STREAM_THRESOLD = 512;
 
     /**
@@ -57,10 +59,6 @@ public class CachedResource extends Resource implements TeeMonitor {
      * Attribute index - The response time of the cached response.
      */
     protected static int ATTR_RESPONSE_TIME = -1;
-    /**
-     * Attribute index - The last modified date of the cached content
-     */
-    protected static int ATTR_LAST_MODIFIED = -1;
     /**
      * Attribute index - Does this cached resource should revalidate.
      */
@@ -89,6 +87,10 @@ public class CachedResource extends Resource implements TeeMonitor {
      * Attribute index - The extension headers attribute.
      */
     protected static int ATTR_EXTRA_HEADERS = -1;
+    /**
+     * Attribute index - The Last modified of this resource
+     */
+    protected static int ATTR_REPLY_LAST_MODIFIED = -1;
     /**
      * Attribute index - The resource store associated with this resource.
      */
@@ -140,11 +142,6 @@ public class CachedResource extends Resource implements TeeMonitor {
 			      , null
 			      , Attribute.COMPUTED);
 	ATTR_RESPONSE_TIME = AttributeRegistry.registerAttribute(c, a);
-	// Declare the last-modified attribute value.
-	a = new LongAttribute("last-modified"
-			      , null
-			      , Attribute.COMPUTED);
-	ATTR_LAST_MODIFIED = AttributeRegistry.registerAttribute(c, a);
 	// Declare the revalidate attribute value.
 	a = new BooleanAttribute("revalidate"
 				 , Boolean.FALSE
@@ -181,6 +178,11 @@ public class CachedResource extends Resource implements TeeMonitor {
 				    , null
 				    , Attribute.COMPUTED);
 	ATTR_EXTRA_HEADERS = AttributeRegistry.registerAttribute(c, a);
+	// The last modified attribute:
+	a = new DateAttribute("reply-last-modified",
+			      null,
+			      Attribute.COMPUTED|Attribute.EDITABLE) ;
+	ATTR_REPLY_LAST_MODIFIED = AttributeRegistry.registerAttribute(c,a);
     }
 
     // Cached Entity tag HTTP list value for this resource.
@@ -189,6 +191,7 @@ public class CachedResource extends Resource implements TeeMonitor {
     CacheFilter   filter  = null;
     
     private boolean invalidated = false;
+    private boolean willRevalidate = false;
 
     /**
      * Get the store associated with this resource.
@@ -337,17 +340,6 @@ public class CachedResource extends Resource implements TeeMonitor {
 	setLong(ATTR_RESPONSE_TIME, responsetime);
     }
 
-    /**
-     * Get the last modified time of that entry.
-     * This usually corresponds to the <code>Last-Modified</code> HTTP header.
-     * @return A long number of milliseconds since java epoch giving the last
-     * modified date of the cached entry, or <strong>-1</strong> if undefined.
-     */
-
-    public long getLastModified() {
-	return getLong(ATTR_LAST_MODIFIED, -1);
-    }
-
     /**
      * Set the last modified time of that cached entry.
      * @param lastmodified The last modification date as a number of 
@@ -356,7 +348,15 @@ public class CachedResource extends Resource implements TeeMonitor {
      */
 
     public void setLastModified(long lastmodified) {
-	setLong(ATTR_LAST_MODIFIED, lastmodified);
+	setLong(ATTR_REPLY_LAST_MODIFIED, lastmodified);
+    }
+    /**
+     * Get this Cached Resource last modification time.
+     * @return A long giving the date of the last modification time, or
+     *    <strong>-1</strong> if undefined.
+     */
+    public long getLastModified() {
+	return getLong(ATTR_REPLY_LAST_MODIFIED, (long) -1) ;
     }
 
     /**
@@ -493,6 +493,22 @@ public class CachedResource extends Resource implements TeeMonitor {
 	setValue(ATTR_EXTRA_HEADERS, a);
     }
 
+    /**
+     * get the revalidation request flag of this resource
+     * @return a boolean, tru if the resource has to be revalidated next call
+     */
+    public boolean getWillRevalidate() {
+	return willRevalidate;
+    }
+
+    /**
+     * get the revalidation request flag of this resource
+     * @return a boolean, tru if the resource has to be revalidated next call
+     */
+    public void setWillRevalidate(boolean doIt) {
+	willRevalidate = doIt;
+    }
+    
     /**
      * Get the cached data for that cached entry.
      * @return A <em>non-buffered</em> output stream.
@@ -564,6 +580,8 @@ public class CachedResource extends Resource implements TeeMonitor {
 	try {
 	    out = new FileOutputStream(getFile());
 	} catch (IOException ex) {
+	    if (debug)
+		ex.printStackTrace();
 	    // We'll let cacheContent take care of that situation:
 	    return null;
 	}
@@ -601,7 +619,7 @@ public class CachedResource extends Resource implements TeeMonitor {
 	    int  total    = 0;
 	    // We may get an IOException if reading from a peer's closed:
 	    try {
-		while ((count = in.read(buffer)) > 0) {
+		while ((count = in.read(buffer)) >= 0) {
 		    out.write(buffer, 0, count);
 		    total += count;
 		}
@@ -609,6 +627,12 @@ public class CachedResource extends Resource implements TeeMonitor {
 		in.close();
 	    } catch (IOException ex) {
 		error = ex;
+		if (debug) {
+		    ex.printStackTrace();
+		    System.out.println("** Got: " + total + 
+				       " last: " + count +
+				       " upnewsize: " + upnewsize);
+		}
 	    }
 	    // Check that we got the right number of bytes:
 	    if ( error != null ) {
@@ -641,16 +665,24 @@ public class CachedResource extends Resource implements TeeMonitor {
      */
 
     public void setResourceHeaders(Request request, Reply reply) {
+	int repstat = reply.getStatus();
 	// Well known headers first:
-	if ( reply.hasHeader(reply.H_CONTENT_TYPE) )
+	if ( reply.hasHeader(reply.H_CONTENT_TYPE)) {
 	    setContentType(reply.getContentType());
-	if ( reply.hasHeader(reply.H_CONTENT_LENGTH) )
+	}
+	if ( reply.hasHeader(reply.H_CONTENT_LENGTH) || 
+	     (repstat != HTTP.NOT_MODIFIED) ) {
 	    setContentLength(reply.getContentLength());
-	if ( reply.hasHeader(reply.H_LAST_MODIFIED) )
+	}
+	if ( reply.hasHeader(reply.H_LAST_MODIFIED) || 
+	     (repstat != HTTP.NOT_MODIFIED) ) {
 	    setLastModified(reply.getLastModified());
-	if ( reply.hasHeader(reply.H_ETAG) ) 
+	}
+	if ( reply.hasHeader(reply.H_ETAG) ) {
 	    setETag(reply.getETag().toString());
-
+	} else if (repstat != HTTP.NOT_MODIFIED) {
+	    setETag(null);
+	}
 	// We hope not to end up here as much as possible:
 	ArrayDictionary a;
 	a = getExtraHeaders();
@@ -690,7 +722,7 @@ public class CachedResource extends Resource implements TeeMonitor {
 	ArrayDictionary a = getExtraHeaders();
 	if ( reply.getStatus() != HTTP.NOT_MODIFIED ) {
 	    int cl = getContentLength();
-	    if ((cl > 0) && (reply.getStatus() != HTTP.PARTIAL_CONTENT))
+	    if ((cl >= 0) && (reply.getStatus() != HTTP.PARTIAL_CONTENT))
 		reply.setContentLength(getContentLength());
 	    reply.setContentType(getContentType());
 	    reply.setLastModified(getLastModified());
@@ -838,7 +870,7 @@ public class CachedResource extends Resource implements TeeMonitor {
 	    && (reply.getStatus() != HTTP.PRECONDITION_FAILED)
 	    && (reply.getStatus() / 100) != 5 /*remote connection failed*/ ) {
 	    int oldsz = getContentLength();
-	    // Save a number of usefull 1.0/1.1 header values:
+	    // Save a number of useful 1.0/1.1 header values:
 	    setStatus(reply.getStatus());
 	    setResourceHeaders(request, reply);
 	    // Try active caching (if reply has a stream):
@@ -992,6 +1024,12 @@ public class CachedResource extends Resource implements TeeMonitor {
 	    setRequestValidation(request);
 	    return false;
 	}
+	// Check for explicit revalidation
+	if ( willRevalidate ) {
+	    willRevalidate = false;
+	    setRequestValidation(request);
+	    return false;
+	}	    
 	// Check freshness:
 	long current_age = computeCurrentAge();
 	long freshness   = getMaxAge();
@@ -1128,6 +1166,8 @@ public class CachedResource extends Resource implements TeeMonitor {
 	    filter.markUsed(this);
 	    return reply;
 	} catch (IOException ex) {
+	    if (debug)
+		ex.printStackTrace();
 	    // Some exception occured, delete that resource (no longer usefull)
 	    delete();
 	}
@@ -1232,6 +1272,7 @@ public class CachedResource extends Resource implements TeeMonitor {
     {
 	slowpickle = false;
 	invalidated = false;
+	willRevalidate = false;
 	setValue(ATTR_IDENTIFIER, request.getURL().toExternalForm());
 	setValue(ATTR_CACHE_FILTER, filter);
 	setInt(ATTR_GENERATION, filter.state.getGeneration());
